#include <clutter/clutter.h>
#include <mx/mx.h>

static gboolean
close_cb(ClutterStage* stage, ClutterEvent* event, gpointer data)
{
    clutter_main_quit();
    return TRUE;
}

static void
run_cb(ClutterText* text, gpointer data)
{
    g_spawn_command_line_async(clutter_text_get_text(text), NULL);
    clutter_text_set_text(text, "");
}

static void
launch_mxlauncher_cb(MxButton* button, gpointer data)
{
    g_spawn_command_line_async("/usr/lib/webkit-clutter-0/libexec/MxLauncher", NULL);
}

int main(int argc, char** argv)
{
    if (clutter_init(&argc, &argv) != CLUTTER_INIT_SUCCESS) {
        g_error("Failed to initialize clutter.");
        return 1;
    }

    MxWindow* window = mx_window_new();
    ClutterStage* stage = mx_window_get_clutter_stage(window);
    clutter_actor_set_size(CLUTTER_ACTOR(stage), 500, 100);

    g_signal_connect(window, "destroy",
                     G_CALLBACK(close_cb), NULL);

    ClutterActor* entry = mx_entry_new();
    clutter_actor_set_width(entry, 400);

    MxToolbar* toolbar = mx_window_get_toolbar(window);

    /* clutter_actor_add_child doesn't seem to work with Mx */
    clutter_container_add_actor(CLUTTER_CONTAINER(toolbar), entry);

    g_signal_connect(mx_entry_get_clutter_text(MX_ENTRY(entry)),
                     "activate", G_CALLBACK(run_cb), NULL);

    ClutterActor* box = mx_box_layout_new();
    mx_box_layout_set_orientation(MX_BOX_LAYOUT(box), MX_ORIENTATION_VERTICAL);

    mx_window_set_child(window, box);

    ClutterActor* button = mx_button_new_with_label("MxLauncher");
    clutter_container_add_actor(CLUTTER_CONTAINER(box), button);

    g_signal_connect(button, "clicked",
                     G_CALLBACK(launch_mxlauncher_cb), NULL);

    GString* env_string = g_string_new("");

    char** environ = g_get_environ();
    for (int i = 0; environ[i] != NULL; i++)
        g_string_append_printf(env_string, "%s\n", environ[i]);
    g_strfreev(environ);

    ClutterActor* scroll_view = mx_scroll_view_new();
    clutter_actor_set_height(scroll_view, 300);
    clutter_container_add_actor(CLUTTER_CONTAINER(box), scroll_view);

    ClutterActor* viewport = mx_viewport_new();
    clutter_container_add_actor(CLUTTER_CONTAINER(scroll_view), viewport);

    ClutterActor* env_label = mx_label_new_with_text(env_string->str);
    clutter_container_add_actor(CLUTTER_CONTAINER(viewport), env_label);

    mx_label_set_line_wrap(MX_LABEL(env_label), TRUE);

    g_string_free(env_string, TRUE);

    mx_window_show(window);

    clutter_main();

    return 0;
}
